import sys
import io

import itertools
import logging
import pandas as pd
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
import re

from .log_config import logger


def emptyDf(size, region, date):
	"""Returns a df full of NA i.e empty df
	size - 200 or 50 i.e. viral 50 or top 200
	region - Region of interest
	date - Date of interest
	"""
	if(size == 50):
		isViral = True
	else:
		isViral = False

	pos = list(itertools.repeat("NA", size))
	track = list(itertools.repeat("NA", size))
	art = list(itertools.repeat("NA", size))
	stre = list(itertools.repeat("NA", size))
	reg = list(itertools.repeat(region, size))
	dat = list(itertools.repeat(date, size))
	i = list(itertools.repeat("NA", size))

	if(isViral):
		empty = pd.DataFrame.from_dict({"position": pos, "track name":track, "artist":art, "region": reg, "date":dat, "spotify_id":i})
	else:
		empty = pd.DataFrame.from_dict({"position": pos, "track name":track, "artist":art, "streams":stre, "region": reg, "date":dat, "spotify_id":i})

	return empty


class SpotifyChartsBase(object):
	def __init__(self):
		""" Set up necessary things
		"""
		self.logger = logger


	def __regex(self, url):
		"""Extract Track ID from URL
		url - Spotify track URL
		"""
		if(type(url) == str):
			sr = re.search(r"https://open.spotify.com/track/(.*)" ,url, re.I|re.M)
			trackId = sr.group(1)
		else:
			trackId = "N/A"
		
		return trackId


	def __makeRequests(self, url, date, region, isSkip, size):
			"""Make the HTTP request, clean data, and return as df
			url - The URL to make request
			isSkip - Whether or not to skip first row of CSV file
			"""
			headers = {"Host":"spotifycharts.com", "User-Agent": "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/69.0.3497.100 Mobile Safari/537.36"}

			retries = Retry(total = 10, backoff_factor = 2, status_forcelist = [500, 502, 503, 504, 404])

			try:
				s = requests.Session()
				s.mount("https://", HTTPAdapter(max_retries = retries))
				res = s.get(url, headers = headers, timeout = 3)

				if(res.status_code == 200):
					if(res.headers["Content-Type"] == "text/html; charset=UTF-8"):
						self.logger.error("***** Data not found. Generating empty dataframe *****")

						df = emptyDf(size, region, date)

					else:
						data = res.content
						if(isSkip):
							df = pd.read_csv(io.StringIO(data.decode("utf-8")), skiprows=1)
						else:
							df = pd.read_csv(io.StringIO(data.decode("utf-8")))

						df["date"] = date
						df["region"] = region
						df["spotify_id"] =  df["URL"].apply(lambda x: self.__regex(x))
						df.drop(["URL"], axis=1, inplace=True)


				else:
					self.logger.error("***** Data not found. Generating empty dataframe *****")
					df = emptyDf(size, region, date)

				return df
			except Exception as e:
				self.logger.error(f"***** {e} Data not found. Generating empty dataframe *****")

				df = emptyDf(size, region, date)
				
				return df


	def helperTop200Weekly(self, date, region):
		"""Base method to return df of top 200 weekly
		date: Date to extract df
		region: Region of interest
		"""
		self.logger.info("Extracting top 200 weekly for {} - {}".format(date, region))
		url = "https://spotifycharts.com/regional/{}/weekly/{}/download".format(region, date)
		data = self.__makeRequests(url, date, region, True, 200)

		return data

	
	def helperTop200Daily(self, date, region):
		"""Base method to return df of top 200 daily
		date: Date to extract df
		region: Region of interest
		"""
		self.logger.info("Extracting top 200 daily for {} - {}".format(date, region))
		url = "https://spotifycharts.com/regional/{}/daily/{}/download".format(region, date)
		data = self.__makeRequests(url, date, region, True, 200)

		return data


	def helperViral50Weekly(self, date, region):
		"""Base method to return df of viral 50 weekly
		date: Date to extract df
		region: Region of interest
		"""
		self.logger.info("Extracting viral 50 weekly for {} - {}".format(date, region))
		url = "https://spotifycharts.com/viral/{}/weekly/{}/download".format(region, date)
		data = self.__makeRequests(url, date, region, False, 50)

		return data


	def helperViral50Daily(self, date, region):
		"""Base method to return df of viral 50 daily
		date: Date to extract df
		region: Region of interest
		"""
		self.logger.info("Extracting viral 50 daily for {} - {}".format(date, region))
		url = "https://spotifycharts.com/viral/{}/daily/{}/download".format(region, date)
		data = self.__makeRequests(url, date, region, False, 50)

		return data

		



import datetime
import sys


from .log_config import logger
from .exceptions import FyChartsException


def defaultListOfDates(isWeekly, isViral):
	viralWeeklyStart = "2017-01-05"
	topWeeklyStart = "2016-12-22"
	allDailyStart = "2017-01-01"

	if(isWeekly):
		if(isViral):
			start = datetime.datetime.strptime(viralWeeklyStart, "%Y-%m-%d")
		else:
			start = datetime.datetime.strptime(topWeeklyStart, "%Y-%m-%d") 
	else:
		start = datetime.datetime.strptime(allDailyStart, "%Y-%m-%d")

	end = datetime.datetime.now()

	dates = [] 
	if(isWeekly): 
		if(isViral):
			gen = [start + datetime.timedelta(weeks = x) for x in range(0, (end-start).days + 1)]
			for date in gen:
				if(date < end):
					dt = date + datetime.timedelta(days = 0)
					dates.append(dt.strftime("%Y-%m-%d"))
		else:
			gen = [start + datetime.timedelta(weeks = x) for x in range(0, (end-start).days + 1)]
			for date in gen:
				if(date<end):
					dt = date + datetime.timedelta(days=1)
					dates.append(dt.strftime("%Y-%m-%d"))

	else:
		gen = [start + datetime.timedelta(days=x) for x in range(0, (end-start).days+1)]
		for date in gen:
			if(date<=end):
				dates.append(date.strftime("%Y-%m-%d"))

	return dates


def returnDatesAndRegions(start=None, end=None, theRegs=None, isWeekly=False, isViral=False):
	"""Return list of dates and regions based on query
	start - String start of range. Can be None i.e. range is from the beginning of data
	end - String end of range. Can be None i.e. range is till today
	isWeekly - Frequency is weekly or daily. Default is False i.e. Frequency is daily
	isViral - Top 200 or Viral 50. Default is False i.e. Return Top 200
	region - Region of chart. Default is None i.e. Get for all regions
	"""
	# Default values
	regions = ["global", "ad", "ar", "at", "au", "be", "bg", "bo", "br", "ca", "ch", "cl", "co", "cr", "cy", "cz", "de", "dk", "do", "ec", "ee", "es", "fi", "fr", "gb", "gr", "gt", "hk", "hn", "hu", "id", "ie", "il", "is", "it", "jp", "lt", "lu", "lv", "mc", "mt", "mx","my", "ni", "nl", "no", "nz", "pa", "pe", "ph", "pl", "pt", "py", "ro", "se", "sg", "sk", "sv", "th", "tr", "tw", "us", "uy", "vn"]
	viralWeeklyStart = "2017-01-05"
	topWeeklyStart = "2016-12-22"
	allDailyStart = "2017-01-01"

	#Required since dates taken are very specific
	defaultList = defaultListOfDates(isWeekly, isViral)
	#--------------------------------------------

	# Helper for Exception handling
	if(isWeekly and isViral):
		func = "viral50Weekly"
	elif(isWeekly and not isViral):
		func = "top200Weekly"
	elif(not isWeekly and isViral):
		func = "viral50Daily"
	elif(not isWeekly and not isViral):
		func = "top200Daily"
	# 

	# Start dates
	if(start is None): #From the beginning
		if(isWeekly):
			if(isViral):
				start = datetime.datetime.strptime(viralWeeklyStart, "%Y-%m-%d")
			else:
				start = datetime.datetime.strptime(topWeeklyStart, "%Y-%m-%d") 
		else:
			start = datetime.datetime.strptime(allDailyStart, "%Y-%m-%d")
	else:
		if(start in defaultList):
			start = datetime.datetime.strptime(start, "%Y-%m-%d")
		else:
			orderedList = sorted(defaultList, key=lambda x: datetime.datetime.strptime(x, "%Y-%m-%d") - datetime.datetime.strptime(start, "%Y-%m-%d"))
			closest = [d for d in orderedList if d >= start]
			suggest = closest[0:5]
			logger.info(f"The start date {start} provided for {func} is invalid. Wanna give one these a try? {suggest}")
			choice = input("Enter (1) to use the first suggestion, or (2) to quit and set yourself: ")
			if(int(choice) == 1):
				start = datetime.datetime.strptime(suggest[0], "%Y-%m-%d")
			elif(int(choice) == 2):
				sys.exit()
			else:
				raise FyChartsException("Invalid Choice.")


	# End dates
	if(end is None): #Up to now
		end = datetime.datetime.now()
	else:
		end = datetime.datetime.strptime(end, "%Y-%m-%d")
		

	# Region
	region = []
	if(theRegs is None):
		region = regions
	else:
		if(type(theRegs) is not list):
			regs = []
			regs.append(theRegs)
			theRegs = regs
			
		for aReg in theRegs:
			if(aReg in regions):
				region.append(aReg)
			else:
				raise FyChartsException(f"Data for the region --> {aReg} <-- requested for {func} does not exist. Please try another region")

	#Generate list of dates
	dates = [] 
	if(isWeekly): 
		if(isViral):
			gen = [start + datetime.timedelta(weeks=x) for x in range(0, (end-start).days+1)]
			for date in gen:
				if(date<end):
					dt = date + datetime.timedelta(days=0)
					dates.append(dt.strftime("%Y-%m-%d"))
		else:
			gen = [start + datetime.timedelta(weeks=x) for x in range(0, (end-start).days+1)]
			for date in gen:
				if(date<end):
					dt = date + datetime.timedelta(days=0)
					dates.append(dt.strftime("%Y-%m-%d"))

	else:
		gen = [start + datetime.timedelta(days=x) for x in range(0, (end-start).days+1)]
		for date in gen:
			if(date<=end):
				dates.append(date.strftime("%Y-%m-%d"))

	var = {"dates": dates, "region": region}
	return var

def whatDates(start, end, desired):
	
	if(desired == "top200Daily"):
		isWeekly = False
		isViral = False
	elif(desired == "top200Weekly"):
		isWeekly = True
		isViral = False
	elif(desired == "viral50Daily"):
		isWeekly = False
		isViral = True
	elif(desired == "viral50Weekly"):
		isWeekly = True
		isViral = True

	allValids = defaultListOfDates(isWeekly, isViral)

	fin = [date for date in allValids if date <= end and date >= start]
	
	return fin


from os import path, remove
import logging

if path.isfile("logs/logs.log"):
    remove("logs/logs.log")
 
# Create the Logger
logger = logging.getLogger('meGlobalLogger')
logger.setLevel(logging.DEBUG)
 

#Handler for console
console_logger = logging.StreamHandler()
 
# Create a Formatter for formatting the log messages
format = logging.Formatter('%(levelname)s : %(asctime)-15s : %(message)s', '%d/%m/%Y %I:%M:%S %p')

# Add the Formatter to the Handlers
console_logger.setFormatter(format)
 
# Add the Handler to the Logger
logger.addHandler(console_logger)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fycharts",
    version="4.0.1",
    author="Kelvin Gakuo",
    author_email="kelvingakuo@gmail.com",
    description="A fully-fledged installable python package for extracting top 200 and viral 50 charts off of spotifycharts.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kelvingakuo/fycharts",
    packages=setuptools.find_packages(),
    install_requires=[
        "pandas==1.1.2",
        "requests==2.24.0",
        "click==7.1.2",
        "colorama==0.4.3"
    ],
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points = """
		[console_scripts]
		fycharts=fycharts.cli:main
	"""
)


import pandas as pd
import requests

from .log_config import logger


def writeToCSV(j, output_file, df):
	if(j == 0):
		isHeaders = True
	else:
		isHeaders = False

	try:
		with open(output_file, 'a', encoding="utf-8") as out:
			logger.info(f"Appending data to the file {output_file}...")
			df.to_csv(out, index = False, header = isHeaders, encoding = "utf-8")
			logger.info(f"Done appending to the file {output_file}!!!")
	except Exception as e:
		raise(e)


def writeToSQLTable(which_data, connector, df):
	""" Writes data to an SQL table depending on what data the user is getting
	"""
	if(which_data == "top200Daily"):
		table = "top_200_daily"
	elif(which_data == "top200Weekly"):
		table = "top_200_weekly"
	elif(which_data == "viral50Daily"):
		table = "viral_50_daily"
	elif(which_data == "viral50Weekly"):
		table = "viral_50_weekly"

	try:
		logger.info(f"Appending data to the table {table}")
		df.to_sql(table, con = connector, if_exists = "append")
		logger.info(f"Done appending to the table {table}!!!")
	except Exception as e:
		raise(e)



def postToRestEndpoint(df, urls, which):
	""" Convert df to json, then post to endpoint
	"""
	fd = df.to_dict(orient = "records")
	dump = {"chart": which, "data": fd}

	if(type(urls) is list):
		urls_ = urls
	else:
		urls_ = []
		urls_.append(urls)

	try:
		for url in urls_:
			logger.info(f"POSTing data to the endpoint {url}")
			requests.post(url, json = dump)
			logger.info("Done POSTing")
	except Exception as e:
		raise(e)


# def pushToCallback(df, callback, which):
# 	fd = df.to_dict(orient = "records")
# 	dump = {"chart": which, "data": fd}

# 	try:
# 		logger.info(f"Pushing data to callback function")
# 		callback(dump)
# 		logger.info("Done pusing")
# 	except Exception as e:
# 		raise(e)


import json
import threading
import re

from queue import Queue

from .crawler_base import SpotifyChartsBase
from .compute_dates import returnDatesAndRegions
from .compute_dates import whatDates
from .log_config import logger

from .write_to_outputs import writeToCSV
from .write_to_outputs import writeToSQLTable
from .write_to_outputs import postToRestEndpoint
# from .write_to_outputs import pushToCallback

from .exceptions import FyChartsException

def validateFile(fileName):
	if(".csv" in fileName):
			return fileName
	else:
		raise FyChartsException("Only CSV files allowed")

# Class definition
class SpotifyCharts(SpotifyChartsBase):
	def __init__(self):
		SpotifyChartsBase.__init__(self)
		self.csv_data_queue = Queue()
		self.db_data_queue = Queue()
		self.post_data_queue = Queue()
		# self.callback_data_queue = Queue()

	def __write_to_csv_from_queue(self, data_q):
		""" Reads a dataframe from the queue, then writes to CSV
		"""
		try:
			work = True
			while work:
				data = data_q.get(block = True)
				if data is None:
					work = False
					return
				else:
					df = data["df"]
					j = data["j"]
					file = data["out_file"]

					writeToCSV(j, file, df)
		except Exception as e:
			raise RuntimeError(e)

	def __write_to_db_from_queue(self, data_q):
		""" Reads a dataframe from the queue, then writes to SQL table
		"""
		try:
			work = True
			while work:
				data = data_q.get(block = True)
				if data is None:
					work = False
					return
				else:
					df = data["df"]
					conn = data["conn"]
					data_type = data["data_type"]

					writeToSQLTable(data_type, conn, df)
		except Exception as e:
			raise RuntimeError(e)


	def __post_to_endpoint_from_queue(self, data_q):
		""" Reads a dataframe from the queue, then POSTs to a REST endpoint
		"""
		try:
			work = True
			while work:
				data = data_q.get(block = True)
				if data is None:
					work = False
					return
				else:
					df = data["df"]
					url = data["url"]
					what_data = data["which_one"]

					postToRestEndpoint(df, url, what_data)
		except Exception as e:
			raise RuntimeError(e)

	# def __pass_to_callback_from_queue(self, data_q):
	# 	""" Reads a dataframe from the queue, then passes it to a callback function
	# 	"""
	# 	try:
	# 		work = True
	# 		while work:
	# 			data = data_q.get(block = True)
	# 			if data is None:
	# 				work = False
	# 				return
	# 			else:
	# 				df = data["df"]
	# 				callback = data["callback"]
	# 				what_data = data["which_one"]

	# 				pushToCallback(df, callback, what_data)
	# 	except Exception as e:
	# 		raise RuntimeError(e)

	def top200Weekly(self, output_file = None, output_db = None, webhook = None, start = None, end = None, region = None):
		"""Write to file the charts data for top 200 weekly
		Params:
			output_file - Name of CSV file to write the data to
			output_db - A connection object to any database supported by SQLAlchemy (https://docs.sqlalchemy.org/en/13/dialects/#included-dialects)
			webhook - A (or a list of) rest endpoint to POST data to 
			start - Start of range (YYYY-MM-DD) as string
			end - End of range (YYYY-MM-DD) as string
			region - Region (or a list of regions) to get data for

		* Any parameter passed as None, means ALL data since the beginning up to now
		"""
		data = returnDatesAndRegions(start, end, region, isWeekly = True, isViral = False)

		dates = data["dates"]
		regions = data["region"]

		if(output_file is not None):
			file = validateFile(output_file)
			a_thread = threading.Thread(target = self.__write_to_csv_from_queue, args = (self.csv_data_queue,))
			a_thread.start()
		if(output_db is not None):
			adb_thread = threading.Thread(target = self.__write_to_db_from_queue, args = (self.db_data_queue,))
			adb_thread.start()
		if(webhook is not None):
			ahook_thread = threading.Thread(target = self.__post_to_endpoint_from_queue, args = (self.post_data_queue,))
			ahook_thread.start()
		# if(callable(callback)):
		# 	# Callback function given
		# 	a_call_thread = threading.Thread(target = self.__pass_to_callback_from_queue, args = (self.callback_data_queue,))
		# 	a_call_thread.start()


		if(output_file is None and output_db is None and webhook is None):
			raise FyChartsException("Please provide at least one output destination")
		
		try:
			j = 0
			while(j < len(dates)):
				if((j + 1) == len(dates)): 
					break
				theRange = dates[j]+"--"+dates[j+1]

				k = j
				for region in regions:
					df = super().helperTop200Weekly(theRange, region)
					if(output_file is not None):
						dict_for_csv = {"df": df, "out_file": file, "j": k}
						self.csv_data_queue.put(dict_for_csv)
					if(output_db is not None):
						dict_for_db = {"df": df, "conn": output_db, "data_type": "top200Weekly"}
						self.db_data_queue.put(dict_for_db)
					if(webhook is not None):
						dict_for_hook = {"df": df, "url": webhook, "which_one": "top_200_weekly"}
						self.post_data_queue.put(dict_for_hook)
					# if(callable(callback)):
					# 	dict_for_callback = {"df": df, "callback": fn, "which_one": "top_200_weekly"}
					# 	self.callback_data_queue.put(dict_for_callback)

					k = k + 1
				j = j + 1
		except Exception as e:
			a_thread.join()
			adb_thread.join()
			ahook_thread.join()
			# a_call_thread.join()
			raise(e)

		self.csv_data_queue.put(None)
		self.db_data_queue.put(None)
		self.post_data_queue.put(None)
		# self.callback_data_queue.put(None)

	def top200Daily(self, output_file = None, output_db = None, webhook = None, start = None, end = None, region = None):
		"""Write to file the charts data for top 200 daily
		Params:
			output_file - Name of CSV file to write the data to
			output_db - A connection object to any database supported by SQLAlchemy (https://docs.sqlalchemy.org/en/13/dialects/#included-dialects
			webhook - A (or a list of) rest endpoint to POST data to 
			start - Start of range (YYYY-MM-DD) as string
			end - End of range (YYYY-MM-DD) as string
			region - Region (or a list of regions) to get data for

		* Any parameter passed as None, means ALL data since the beginning up to now
		"""
		data = returnDatesAndRegions(start, end, region, isWeekly = False, isViral = False)

		dates = data["dates"]
		regions = data["region"]

		if(output_file is not None):
			file = validateFile(output_file)
			b_thread = threading.Thread(target = self.__write_to_csv_from_queue, args = (self.csv_data_queue,))
			b_thread.start()
		if(output_db is not None):
			bdb_thread = threading.Thread(target = self.__write_to_db_from_queue, args = (self.db_data_queue,))
			bdb_thread.start()
		if(webhook is not None):
			bhook_thread = threading.Thread(target = self.__post_to_endpoint_from_queue, args = (self.post_data_queue,))
			bhook_thread.start()


		if(output_file is None and output_db is None):
			raise FyChartsException("Please provide at least one output destination")

		j = 0
		while(j < len(dates)):
			theRange = dates[j]

			k = j
			for region in regions:
				df = super().helperTop200Daily(theRange, region)
				if(output_file is not None):
					dict_for_csv = {"df": df, "out_file": file, "j": k}
					self.csv_data_queue.put(dict_for_csv)
				if(output_db is not None):
					dict_for_db = {"df": df, "conn": output_db, "data_type": "top200Daily"}
					self.db_data_queue.put(dict_for_db)
				if(webhook is not None):
					dict_for_hook = {"df": df, "url": webhook, "which_one": "top_200_daily"}
					self.post_data_queue.put(dict_for_hook)
				k = k + 1

			j = j + 1
		self.csv_data_queue.put(None)
		self.db_data_queue.put(None)
		self.post_data_queue.put(None)

	def viral50Weekly(self, output_file = None, output_db = None, webhook = None, start = None, end = None, region = None):
		"""Write to file the charts data for viral 50 weekly
		Params:
			output_file - Name of CSV file to write the data to
			output_db - A connection object to any database supported by SQLAlchemy (https://docs.sqlalchemy.org/en/13/dialects/#included-dialects
			webhook - A (or a list of) rest endpoint to POST data to 
			start - Start of range (YYYY-MM-DD) as string
			end - End of range (YYYY-MM-DD) as string
			region - Region (or a list of regions) to get data for

		* Any parameter passed as None, means ALL data since the beginning up to now
		"""
		data = returnDatesAndRegions(start, end, region, isWeekly = True, isViral = True)

		dates = data["dates"]
		regions = data["region"]

		if(output_file is not None):
			file = validateFile(output_file)
			c_thread = threading.Thread(target = self.__write_to_csv_from_queue, args = (self.csv_data_queue,))
			c_thread.start()
		if(output_db is not None):
			cdb_thread = threading.Thread(target = self.__write_to_db_from_queue, args = (self.db_data_queue,))
			cdb_thread.start()
		if(webhook is not None):
			chook_thread = threading.Thread(target = self.__post_to_endpoint_from_queue, args = (self.post_data_queue,))
			chook_thread.start()


		if(output_file is None and output_db is None):
			raise FyChartsException("Please provide at least one output destination")

		j = 0
		while(j < len(dates)):
			if((j) == len(dates)): 
				break
			theRange = dates[j]+"--"+dates[j]

			k = j
			for region in regions:
				df = super().helperViral50Weekly(theRange, region)
				if(output_file is not None):
					dict_for_csv = {"df": df, "out_file": file, "j": k}
					self.csv_data_queue.put(dict_for_csv)
				if(output_db is not None):
					dict_for_db = {"df": df, "conn": output_db, "data_type": "viral50Weekly"}
					self.db_data_queue.put(dict_for_db)
				if(webhook is not None):
					dict_for_hook = {"df": df, "url": webhook, "which_one": "viral_50_weekly"}
					self.post_data_queue.put(dict_for_hook)
				k = k + 1

			j = j + 1
		self.csv_data_queue.put(None)
		self.db_data_queue.put(None)
		self.post_data_queue.put(None)

	def viral50Daily(self, output_file = None, output_db = None, webhook = None, start = None, end = None, region = None):
		"""Write to file the charts data for viral 50 daily
		Params:
			output_file - Name of CSV file to write the data to
			output_db - A connection object to any database supported by SQLAlchemy (https://docs.sqlalchemy.org/en/13/dialects/#included-dialects
			webhook - A (or a list of) rest endpoint to POST data to 
			start - Start of range (YYYY-MM-DD) as string
			end - End of range (YYYY-MM-DD) as string
			region - Region (or a list of regions) to get data for

		* Any parameter passed as None, means ALL data since the beginning up to now
		"""
		data = returnDatesAndRegions(start, end, region, isWeekly = False, isViral = True)

		dates = data["dates"]
		regions = data["region"]

		if(output_file is not None):
			file = validateFile(output_file)
			d_thread = threading.Thread(target = self.__write_to_csv_from_queue, args = (self.csv_data_queue,))
			d_thread.start()
		if(output_db is not None):
			ddb_thread = threading.Thread(target = self.__write_to_db_from_queue, args = (self.db_data_queue,))
			ddb_thread.start()
		if(webhook is not None):
			dhook_thread = threading.Thread(target = self.__post_to_endpoint_from_queue, args = (self.post_data_queue,))
			dhook_thread.start()


		if(output_file is None and output_db is None):
			raise FyChartsException("Please provide at least one output destination")

		j = 0
		while(j < len(dates)):
			theRange = dates[j]

			k = j
			for region in regions:
				df = super().helperViral50Daily(theRange, region)
				if(output_file is not None):
					dict_for_csv = {"df": df, "out_file": file, "j": k}
					self.csv_data_queue.put(dict_for_csv)
				if(output_db is not None):
					dict_for_db = {"df": df, "conn": output_db, "data_type": "viral50Daily"}
					self.db_data_queue.put(dict_for_db)
				if(webhook is not None):
					dict_for_hook = {"df": df, "url": webhook, "which_one": "viral_50_daily"}
					self.post_data_queue.put(dict_for_hook)

				k = k + 1

			j = j + 1
		self.csv_data_queue.put(None)
		self.db_data_queue.put(None)
		self.post_data_queue.put(None)

	# ====== UTILITY FUNCTIONS ======
	def validDates(self, start, end, desired):
		""" Returns a table of valid dates from a start date to an end date provided, for the specific data desired
		Params:
			start - Start date of range (YYYY-MM-DD) as string
			end - End date of range (YYYY-MM-DD) as string
			desired - A string specifying the kind of data desired
				Accepts:
					* top200Daily
					* top200Weekly
					* viral50Daily
					* viral50Weekly
		"""
		theDates = whatDates(start, end, desired)
		print(json.dumps(theDates, indent = 4))


		


		









import click

from fycharts.SpotifyCharts import SpotifyCharts

@click.command()
@click.option("--start", "-s", help = "Start date of chart (YYYY-MM-DD)")
@click.option("--end", "-e", help = "End date of chart (YYYY-MM-DD)")
@click.option("--region", "-r", multiple = True, help = "The region(s) to get the chart for")
@click.option("--csv", "-c", default = '', help = "CSV file to output to")
@click.option("--webhook", "-w", multiple = True, help = "Webhook(s) to POST the data to ")
@click.argument("chart")
def main(chart, start, end, csv, region, webhook):
	""" CLI version of fycharts (https://pypi.org/project/fycharts/)
	
	fycharts [OPTIONS] chartName

	===== EXAMPLES

	fycharts --csv out.csv viral50Weekly
	
	fycharts --start 2020-05-06 --end 2020-10-10 --csv out.csv viral50Weekly
	
	fycharts --region US viral50Weekly
	
	fycharts -r US -r UK viral50Weekly
	
	fycharts --webhook https://mywebhook.site/1232/post viral50Weekly
	
	fycharts -w https://mywebhook.site/1/post -w https://mywebhook.site/2/post viral50Weekly
	
	======
	"""
	region = list(region)
	webhook = list(webhook)

	if(len(region) == 0):
		region = None

	if(len(webhook) == 0 and csv == ''):
		click.secho("ERROR: Please set at least one output mode (CSV or webhooks)", fg = "red", bold = True)
	else:																																		
		api = SpotifyCharts()
		chart_choices = ["top200Weekly", "top200Daily", "viral50Weekly", "viral50Daily"]

		if(chart == "top200Weekly"):
			click.secho("Extracting top200Weekly...", fg = "cyan", bold = True)
			api.top200Weekly(csv, None, webhook, start, end, region)
			click.secho("Extraction complete\n", fg = "white", bold = True)
		elif(chart == "top200Daily"):
			click.secho("Extracting top200Daily...", fg = "cyan", bold = True)
			api.top200Daily(csv, None, webhook, start, end, region)
			click.secho("Extraction complete\n", fg = "white", bold = True)
		elif(chart == "viral50Weekly"):
			click.secho("Extracting viral50Weekly...", fg = "cyan", bold = True)
			api.viral50Weekly(csv, None, webhook, start, end, region)
			click.secho("Extraction complete\n", fg = "white", bold = True)
		elif(chart == "viral50Daily"):
			click.secho("Extracting viral50Daily...", fg = "cyan", bold = True)
			api.viral50Daily(csv, None, webhook, start, end, region)
			click.secho("Extraction complete\n", fg = "white", bold = True)
		else:
			click.secho(f"ERROR: {chart} could not be processed. Select either of: {', '.join(chart_choices)}", fg = "red", bold = True)